@@ -269,7 +269,9 @@ fn parse(text: &str) -> Parse {
         }
     }
 
-    let mut tokens = lex(text);
+    let mut tokens = lex(text)
+        .map(|(k, t)| (k, t.to_string()))
+        .collect::<Vec<_>>();
     tokens.reverse();
     Parser {
         tokens,
@@ -706,7 +708,7 @@ impl Paragraph {
     }
 
     /// Returns an iterator over all values for the given key in the paragraph.
-    pub fn get_all<'a>(&'a self, key: &'a str) -> impl Iterator<Item = String> + '_ {
+    pub fn get_all<'a>(&'a self, key: &'a str) -> impl Iterator<Item = String> + 'a {
         self.items()
             .filter_map(move |(k, v)| if k.as_str() == key { Some(v) } else { None })
     }
@@ -905,6 +907,8 @@ impl Entry {
                     .collect::<String>();
                 let formatted = format_value(self.key().as_ref().unwrap(), &concat);
                 crate::lex::lex_inline(&formatted)
+                    .map(|(k, t)| (k, t.to_string()))
+                    .collect::<Vec<_>>()
             } else {
                 content
                     .into_iter()
